package org.apache.hadoop.dfs;

import java.io.IOException;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.ipc.RPC;
import org.apache.hadoop.ipc.RPC.Server;

/**
 * This is the per-rack replication controller. It implements 
 * ShuffleFlowReportProtocol to interface with TaskTrackers, and uses
 * DFSClient to interface with DataNodes.
 * 
 * @author sko
 *
 */
public class ReplicationController implements TT2RCProtocol {
  public static final Log LOG =
    LogFactory.getLog("org.apache.hadoop.dfs.ReplicationController");
  public static final int DEFAULT_RC_PORT = 6000;
  
  // Three replication policies
  // local node, remote rack, ...
  public static final int RC_NORM_REP = 0;
  // remote node in the same rack, remote rack, ...
  public static final int RC_REMOTE_REP = 1;
  // local node, remote node in the same rack, ...
  public static final int RC_LOCAL_REP = 2;
  // remote node in the same rack, remote rack, ...
  // But just send data without writing to the disk
  public static final int RC_REMOTE_NW = 3;
  // normal replication without writing to the disk
  public static final int RC_NORM_NW = 4;
  // RC_REMOTE_REP with slow node avoidance
  public static final int RC_REMOTE_AV = 5;
  // RC_NORM_REP with slow node avoidance
  public static final int RC_NORM_AV = 6;

  
  private HashMap<String, NetFlowTable> hostToFlowTable;
  private Server rcServer;
  private String rackName;
  private static int DEFAULT_HANDLER_COUNT = 10;
  //private DistributedFileSystem fs;
  
  public ReplicationController(String rack,
      String rcHost, int rcPort, Configuration conf) throws IOException {
    this.hostToFlowTable = new HashMap<String, NetFlowTable>();
    this.rackName = rack;
    //this.fs = (DistributedFileSystem) FileSystem.get(conf);
    this.rcServer = RPC.getServer(this, rcHost, rcPort,
        conf.getInt("dfs.replication.controller.handler.count",
            DEFAULT_HANDLER_COUNT), false, conf);
    LOG.info("sko: starting replication controller server");
    this.rcServer.start();
  }
  
  public long getProtocolVersion(String protocol, long clientVersion)
      throws IOException {
    return versionID;
  }

  public void flowUpdate(String host, NetFlowTable flowTable) {
    LOG.debug("sko: got " + flowTable + " from " + host);
    synchronized (hostToFlowTable) {
      hostToFlowTable.put(host, flowTable);
    }
  }
  
  public void shutDown() {
    if (rcServer != null)
      rcServer.stop();
  }
}